package dao

import (
	"enterprise/common/model"
	"gorm.io/gorm"
	"time"
)

type StaffSalaryDao struct {
}

func NewStaffSalaryDao() *StaffSalaryDao {
	return &StaffSalaryDao{}
}

func (d *StaffSalaryDao) TableName() string {
	return "staff_salary"
}

func (d *StaffSalaryDao) Create(o *model.StaffSalary) (int64, error) {
	o.CreateTime = time.Now().Unix()
	res := GetDB().Table(d.TableName()).Create(o)
	return o.Id, res.Error
}

func (d *StaffSalaryDao) Update(o *model.StaffSalary) error {
	o.UpdateTime = time.Now().Unix()
	tx := GetDB().Table(d.TableName())
	res := tx.Save(o)
	return res.Error
}

func (d *StaffSalaryDao) Delete(id int64) error {
	res := GetDB().Table(d.TableName()).Delete(&model.StaffSalary{}, id)
	return res.Error
}

func (d *StaffSalaryDao) Get(id int64) (*model.StaffSalary, error) {
	var u model.StaffSalary
	tx := GetDB().Table(d.TableName())
	tx = tx.Where("id = ?", id)
	res := tx.First(&u)
	if res.Error == gorm.ErrRecordNotFound {
		return nil, nil
	}

	if res.Error != nil {
		return nil, res.Error
	}
	return &u, nil
}

func (d *StaffSalaryDao) GetBy(corpId, userId int64, month string) (*model.StaffSalary, error) {
	var u model.StaffSalary
	tx := GetDB().Table(d.TableName())
	tx = tx.Where("corp_id = ?", corpId)
	tx = tx.Where("user_id = ?", userId)
	tx = tx.Where("month = ?", month)
	res := tx.First(&u)
	if res.Error == gorm.ErrRecordNotFound {
		return nil, nil
	}

	if res.Error != nil {
		return nil, res.Error
	}
	return &u, nil
}

func (d *StaffSalaryDao) Query(corpId, userId int64, month string) ([]*model.StaffSalary, error) {
	var u []*model.StaffSalary
	tx := GetDB().Table(d.TableName())

	tx.Where("corp_id = ?", corpId)

	if month != "" {
		tx = tx.Where("month = ?", month)
	}
	if userId != 0 {
		tx.Where("user_id = ?", userId)
	}

	tx.Order("month DESC")

	res := tx.Find(&u)
	if res.Error == gorm.ErrRecordNotFound {
		return nil, nil
	}

	if res.Error != nil {
		return nil, res.Error
	}
	return u, nil
}
